package com.oracle.coherence.handson.chat;

import com.tangosol.net.*;

import java.io.*;
import com.tangosol.util.UUID;
import com.tangosol.util.filter.*;
import com.tangosol.util.MapEvent;
import com.tangosol.util.AbstractMapListener;

import java.util.Date;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

public class ChatClient {

    public static void main(String[] args) {
      String userName = null; 
      String message;
      NamedCache chatmembers = null;
      
       try {
       
           System.out.println("Welcome to the Coherence Chat Client");
           System.out.println("------------------------------------");
           
           BufferedReader console = new BufferedReader(new InputStreamReader(System.in));
           
           System.out.print("User Name:");
           userName = console.readLine();

           // join the chatroom named cache as storage enabled = true
           NamedCache cache = CacheFactory.getCache("chatroom");
           chatmembers = CacheFactory.getCache("chatmembers");
           
           chatmembers.put(userName,userName);
           
           // register a listener to display the messages
           cache.addMapListener(new AbstractMapListener() {
                   public void entryInserted(MapEvent event) {
                      ChatMessage msg = (ChatMessage)event.getNewValue();
                                                          
                      System.out.println("From: " + msg.getFrom());
                      System.out.println("Time: " + new Date(msg.getEntryTime()));
                      System.out.println("Mesg: " + msg.getMessage() );
                      
                      System.out.println();
                    } }
                    , 
                    new MapEventFilter(MapEvent.ENTRY_INSERTED, new NotEqualsFilter("getFrom", userName)),
		    false);
           
           chatmembers.addMapListener( new AbstractMapListener() {
                 public void entryDeleted(MapEvent event) {
                     String who = (String)event.getOldValue();
                     System.out.println(who + " has left the chat");
                 }
                 
                 public void entryInserted(MapEvent event) {
                     String who = (String)event.getNewValue();
                     System.out.println(who + " has entered the chat");
                 }
              }
           );
           
           do {
               System.out.print("\nEnter message or bye to quit: ");
               message = console.readLine();
               
               if ("bye".equals(message))
                  break;
               // else add this to the chat 
               else if ("help".equals(message)) {
                   System.out.println("HELP:");
                   System.out.println("bye - quit");
                   System.out.println("who - list of users in the chat\n");
               } 
               else if ("who".equals(message)) {
                   System.out.println("Current chat memebers");
                   System.out.println("=====================");
                   Set s = chatmembers.entrySet();
                   for (Iterator<Map.Entry> entries = chatmembers.entrySet().iterator() ; entries.hasNext();) {
                       Map.Entry entry = entries.next();
                       String member = (String)entry.getValue();
                       
                       System.out.println(member);    
                   }
               }
               else {            
                   
                   cache.put(new UUID(),new ChatMessage(userName, message));
                   
               }
               
           } while (true);
           
           System.out.println("Bye");
       } 
       catch (Exception e) {
           e.printStackTrace();
       }
       finally {
           chatmembers.remove(userName);
       }

    }
}
